--  <Scripts>
--      <OnLoad function="BGCF_OnLoad"/>
--      <OnClick function="BGCF_OnClick" />
--      <OnDragStart function="BGCF_OnDragStart" />
--      <OnDragStop function="BGCF_OnDragStop" />
--  </Scripts>


local P = {
    frameName="BG_ControlNode",
    }

DB = {}
--      .node[internalID]
--              [icon]
--              [name]
--              [d]
--              [h]
--              [ratio]
--          .damage
--          .healers
--          .fhealers
--          .fdamage
--      .player[guid]
--          .spec=specID
--          .node=internalNodeID
--          .isClose=bool
--          .range=mapUnit(?) --TODO: Make this yards!
--          .timestamp=time

--====================================================================================
-- HOOKED FUNCTIONS

function P.OnEvent(self, event, ...)
    if "ADDON_LOADED" == event then
        P.Addon_Loaded(self, ...);
    end
end


-- Did this so all functions could be "private"
BG_ControlNode:SetScript("OnEvent",P.OnEvent);
BG_ControlNode:RegisterEvent("ADDON_LOADED");

--=============================================

function P.BattlegroundStarted(mapID)
    P.InitDB()
    P.ResetNodeText()
    P.HideAllRows();
    _G[P.frameName]:Show();
    BG_ControlCommon.Print("Starting BG: "..tostring(mapID))
end

function P.BattlegroundEnded()
    DB = {}
    _G[P.frameName]:Hide()
end

function P.NodeUpdate(internalID,textureIndex,graveyardID,areaID,poiID,isObjectIcon,atlasIcon,name,description)
    if not internalID then return; end
    local needsSort;
    if not DB.node[internalID] then
        DB.node[internalID] = P.GetNewRow();
        needsSort = true;
    end
    local row = DB.node[internalID];

    P.WorldMap_SetIcon(row["icon"],textureIndex,graveyardID,areaID,poiID,isObjectIcon,atlasIcon)
    row["name"]:SetText(tostring(name))

    if needsSort then
        -- P.SortRows();
    end
end

function P.EnemyAt(guid,internalNodeID,isClose,range,time)
    -- Sanity...
    if not guid or not internalNodeID then return; end

    -- If we don't know about this node but we're getting updates, something is wrong... bail
    if not DB.node[internalNodeID] then return; end

    local node = DB.node[internalNodeID];

    if not DB.player[guid] then DB.player[guid] = {}; end

    local player = DB.player[guid];

    player.isClose = isClose;
    player.range = range;
    player.timestamp = time;

    if not player.node or player.node ~= internalNodeID then
        local isHealer = BG_ControlCommon.IsHealer(player.spec);
        P.AddPlayerToNode(internalNodeID,isHealer);
        P.RemovePlayerFromNode(player.node,isHealer);
        player.node = internalNodeID;
    end
end


function P.FriendlyAt(guid,internalNodeID,isClose,range,time)
    -- Sanity...
    if not guid or not internalNodeID then return; end

    -- If we don't know about this node but we're getting updates, something is wrong... bail
    if not DB.node[internalNodeID] then return; end

    local node = DB.node[internalNodeID];

    if not DB.player[guid] then DB.player[guid] = {}; end

    local player = DB.player[guid];

    player.isClose = isClose;
    player.range = range;
    player.timestamp = time;
    player.friend = true

    if not player.node or player.node ~= internalNodeID then
        local isHealer = BG_ControlCommon.IsHealer(player.spec);
        P.AddFriendlyToNode(internalNodeID,isHealer);
        P.RemoveFriendlyFromNode(player.node,isHealer);
        player.node = internalNodeID;
    end
end

function P.PlayerDied(guid,flags,time)
    -- If we don't know about the player, we don't care they died
    if not guid or not DB.player[guid] then return; end

    local player = DB.player[guid];

    -- If this is newer data, and we know they're at a node, get rid of em-
    if player.timestamp < time and player.node then
        if BG_ControlCommon.IsHostile(flags) then
            if player.friend then
                P.RemoveFriendlyFromNode(player.node,BG_ControlCommon.IsHealer(player.spec))
            else
                P.RemovePlayerFromNode(player.node,BG_ControlCommon.IsHealer(player.spec));
            end
            player.node = nil;
            player.isClose = nil;
            player.range = 0;
            player.timestamp = timestamp
        else
        end
    end

end


function P.PlayerSpec(guid,specID,timestamp)
    -- Sanity...
    if not guid or not specID then return; end
    if not DB.player[guid] then DB.player[guid] = {}; end
    local pDB = DB.player[guid]

    -- This is creating a spec, and put them in the right place
    if pDB.node and not pDB.spec then
        local isHealer = BG_ControlCommon.IsHealer(specID);

        -- If they are a healer, we need to move them from damage to healing
        if isHealer then
            P.AddPlayerToNode(pDB.node,isHealer);
            P.RemovePlayerFromNode(pDB.node);
        end
    end
    pDB.spec = specID

end

function P.Addon_Loaded(self,...)
    local name = ...
    if "BG_Control" == name then
        self:Hide();
        self:UnregisterEvent("ADDON_LOADED");
        self:RegisterForDrag("RightButton");

        self:SetScript("OnDragStart", function(self) self:StartMoving(); end)
        self:SetScript("OnDragStop", function(self) self:StopMovingOrSizing(); end)

        hooksecurefunc("BG_ControlInterface_BattlegroundStarted", P.BattlegroundStarted);
        hooksecurefunc("BG_ControlInterface_BattlegroundEnded",P.BattlegroundEnded);
        hooksecurefunc("BG_ControlInterface_NodeUpdate",P.NodeUpdate);
        hooksecurefunc("BG_ControlInterface_EnemyDamageAt",P.EnemyAt);
        hooksecurefunc("BG_ControlInterface_EnemyHealerAt",P.EnemyAt);
        hooksecurefunc("BG_ControlInterface_PlayerDied",P.PlayerDied);
        hooksecurefunc("BG_ControlInterface_NumbNodes",P.NumbNodes);
        hooksecurefunc("BG_ControlInterface_PlayerSpec",P.PlayerSpec);
        hooksecurefunc("BG_ControlInterface_FriendlyPlayerAt",P.FriendlyAt);

        -- Cleanup
        P.Addon_Loaded = nil
    end
end

--====================================================================================
-- BEGIN

function P.NumbNodes(quantity)
    for i=1,quantity do
            P.ShowRow(i)
    end
    for i=quantity+1,25 do
        P.HideRow(i)
    end
end


function P.AddFriendlyToNode(internalID,isHealer)
    if not internalID or not DB.node[internalID] then return; end
    local node = DB.node[internalID];
    if isHealer then
        node.fhealers = ( node.fhealers or 0 ) + 1
    else
        node.fdamage = ( node.fdamage or 0 ) + 1
    end
    P.UpdateFriendlyNode(internalID)


end

function P.RemoveFriendlyFromNode(internalID,isHealer)
    if not internalID or not DB.node[internalID] then return; end
    local node = DB.node[internalID];

    if isHealer then
        node.fhealers = ( node.fhealers or 1 ) - 1
    else
        node.fdamage = ( node.fdamage or 1 ) - 1
    end
    P.UpdateFriendlyNode(internalID)
end

function P.UpdateFriendlyNode(internalID)
    if not internalID or not DB.node[internalID] then return; end
    local node = DB.node[internalID];

    local button = node["ratio"]
    local numer = ( node.fhealers or 0 ) +  ( node.fdamage or 0 )
    local denom = ( node.healers or 0 ) + ( node.damage or 0 )
    local result = numer / denom
    if result == math.huge then
        result = string.char(226,136,158)
        P.SetColorByPercent(button,math.huge)
        P.SetColorByPercent(node["name"],math.huge)
    else
        result = math.floor(result*100)
        P.SetColorByPercent(button,result)
        P.SetColorByPercent(node["name"],result)
    end
    button:SetText(tostring(result).."%")
end


function P.SetColorByPercent(button,percent)
        if percent >= 120 then
            button:SetTextColor(0,1,0)
        elseif percent > 80 then
            button:SetTextColor(1,1,0)
        else
            button:SetTextColor(1,0,0)
        end
end

-- TODO: these two functions are the same... except a minus vs plus
function P.AddPlayerToNode(internalID,isHealer)
    -- If it's broken, bail
    if not internalID or not DB.node[internalID] then return; end
    local node = DB.node[internalID];

    local button;
    local number
    if isHealer then
        node.healers = ( node.healers or 0 ) + 1
        number = node.healers
        button = node["h"];
    else
        node.damage =  ( node.damage or 0 ) + 1
        number = node.damage
        button = node["d"];
    end
    button:SetText(tostring(number))
end

function P.RemovePlayerFromNode(internalID,isHealer)
    -- If it's broken, bail
    if not internalID or not DB.node[internalID] then return; end
    local node = DB.node[internalID];

    local button;
    local number
    if isHealer then
        node.healers = ( node.healers or 1 ) - 1
        number = node.healers
        button = node["h"];
    else
        node.damage =  ( node.damage or 1 ) - 1
        number = node.damage
        button = node["d"];
    end
    button:SetText(tostring(number))
end


function P.ResetNodeText()
    for i=1,25 do
        local t = P.GetRowByIndex(i)
        t["name"]:SetText("Unknown")
        t["d"]:SetText("0")
        t["h"]:SetText("0")
        t["ratio"]:SetText("0")
        t.healers = 0
        t.damage = 0
        t.fhealers = 0
        t.fdamage = 0
    end
end


function P.GetNewRow()
    local nextNumber = ( DB.nodeSize or 0 ) + 1;
    _G[P.frameName..nextNumber]:Show();
    DB.nodeSize = nextNumber
    return P.GetRowByIndex(nextNumber);
end

function P.GetRowByIndex(index)
    return {
        ["icon"]=_G[P.frameName..index.."_Icon"],
        ["name"]=_G[P.frameName..index.."_Name"],
        ["d"]=_G[P.frameName..index.."_D"],
        ["h"]=_G[P.frameName..index.."_H"],
        ["ratio"]=_G[P.frameName..index.."_Ratio"]
        }

end

function P.HideAllRows()
    for i = 1,25 do
        P.HideRow(i)
    end
end


function P.HideRow(numb)
    _G[P.frameName..numb]:Hide();
end

function P.ShowRow(numb)
    _G[P.frameName..numb]:Show();
end

function P.WorldMap_ResetPOI(button, isObjectIcon, atlasIcon)
    if (atlasIcon) then
        button:SetAtlas(atlasIcon, true);
    elseif (isObjectIcon == true) then
        button:SetTexture("Interface\\Minimap\\ObjectIcons");
    else
        button:SetTexture("Interface\\Minimap\\POIIcons");
    end
    button:SetPoint("CENTER", 0, 0);
end


function P.WorldMap_SetIcon(button,textureIndex,graveyardID,areaID,poiID,isObjectIcon,atlasIcon)
    P.WorldMap_ResetPOI(button, isObjectIcon, atlasIcon);


    if (not atlasIcon) then
        local x1, x2, y1, y2;
        if (isObjectIcon == true) then
            x1, x2, y1, y2 = GetObjectIconTextureCoords(textureIndex);
        else
            x1, x2, y1, y2 = GetPOITextureCoords(textureIndex);
        end
        button:SetTexCoord(x1, x2, y1, y2);
    else
        button:SetTexCoord(0, 1, 0, 1);
    end
end

function P.InitDB()
    DB = {}
    DB.node = {}
    DB.player = {}

end

function P.ToString(number)
    local retval = tostring(number);
    if retval == "INF" then
        retval = "\u221e"
    end
    return retval;
end